"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const validate_1 = require("@code-engine/validate");
const workers_1 = require("@code-engine/workers");
const events_1 = require("events");
const ono_1 = require("ono");
const os = require("os");
const build_pipeline_1 = require("./build/build-pipeline");
const log_emitter_1 = require("./log-emitter");
const normalize_plugin_1 = require("./plugins/normalize-plugin");
const plugin_controller_1 = require("./plugins/plugin-controller");
const instances = [];
/**
 * The main CodeEngine class.
 */
class CodeEngine extends events_1.EventEmitter {
    constructor(config = {}) {
        super();
        /** @internal */
        this._isDisposed = false;
        this._config = {
            cwd: config.cwd || process.cwd(),
            concurrency: validate_1.validate.number.integer.positive(config.concurrency, "concurrency", os.cpus().length),
            watchDelay: validate_1.validate.number.integer.positive(config.watchDelay, "watchDelay", 300),
            dev: config.dev === undefined ? process.env.NODE_ENV === "development" : config.dev,
            debug: config.debug === undefined ? Boolean(process.env.DEBUG) : config.debug,
        };
        this._logger = new log_emitter_1.LogEmitter(this, this._config.debug);
        this._buildPipeline = new build_pipeline_1.BuildPipeline();
        this._buildPipeline.on("buildStarting" /* BuildStarting */, this.emit.bind(this, "buildStarting" /* BuildStarting */));
        this._buildPipeline.on("buildFinished" /* BuildFinished */, this.emit.bind(this, "buildFinished" /* BuildFinished */));
        this._buildPipeline.on("error" /* Error */, this._crash.bind(this));
        this._workerPool = new workers_1.WorkerPool(this._config.concurrency, this._createContext());
        this._workerPool.on("error" /* Error */, this._crash.bind(this));
        instances.push(this);
    }
    /**
     * Returns all CodeEngine instances that have been created and not yet disposed.
     */
    static get instances() {
        return instances.slice();
    }
    /**
     * Indicates whether the `dispose()` method has been called.
     * Once disposed, a CodeEngine instance is no longer usable.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * Loads one or more CodeEngine plugins.
     */
    async use(...plugins) {
        this._assertNotDisposed();
        for (let pluginDefinition of plugins) {
            let defaultName = `Plugin ${this._buildPipeline.size + 1}`;
            let plugin = await normalize_plugin_1.normalizePlugin(pluginDefinition, this._workerPool, defaultName);
            let controller = new plugin_controller_1.PluginController(plugin);
            this._buildPipeline.add(controller);
        }
    }
    /**
     * Imports one or more JavaScript modules in all worker threads.
     */
    async import(...modules) {
        this._assertNotDisposed();
        for (let moduleDefinition of modules) {
            await this._workerPool.importModule(moduleDefinition);
        }
    }
    /**
     * Deletes the contents of the destination(s).
     */
    async clean() {
        this._assertNotDisposed();
        let context = this._createContext();
        return this._buildPipeline.clean(context);
    }
    /**
     * Runs a full build of all source files.
     */
    async build() {
        this._assertNotDisposed();
        let context = this._createContext();
        return this._buildPipeline.build(this._config.concurrency, context);
    }
    /**
     * Watches source files for changes and runs incremental re-builds whenever changes are detected.
     */
    watch() {
        this._assertNotDisposed();
        let context = this._createContext();
        let { watchDelay, concurrency } = this._config;
        this._buildPipeline.watch(watchDelay, concurrency, context);
    }
    /**
     * Releases system resources that are held by this CodeEngine instance.
     * Once `dispose()` is called, the CodeEngine instance is no longer usable.
     */
    async dispose() {
        if (this._isDisposed) {
            return;
        }
        this._isDisposed = true;
        let index = instances.indexOf(this);
        index >= 0 && instances.splice(index, 1);
        let context = this._createContext();
        await Promise.all([
            this._buildPipeline.dispose(context),
            this._workerPool.dispose(),
        ]);
    }
    /**
     * Disposes all CodeEngine instances.
     */
    // tslint:disable-next-line: member-ordering
    static async disposeAll() {
        await Promise.all(instances.splice(0, instances.length).map((engine) => engine.dispose()));
    }
    /**
     * Returns a string representation of the CodeEngine instance.
     */
    toString() {
        if (this._isDisposed) {
            return `CodeEngine (disposed)`;
        }
        else {
            return `CodeEngine (${this._buildPipeline.size} plugins)`;
        }
    }
    /**
     * Returns the name to use for `Object.toString()`.
     */
    get [Symbol.toStringTag]() {
        return "CodeEngine";
    }
    /**
     * Creates a `Context` object for this CodeEngine instance.
     * @internal
     */
    _createContext() {
        let { cwd, concurrency, dev, debug } = this._config;
        let logger = this._logger;
        return { cwd, concurrency, dev, debug, logger };
    }
    /**
     * Throws an error if the `CodeEngine` has been disposed.
     * @internal
     */
    _assertNotDisposed() {
        if (this.isDisposed) {
            throw ono_1.ono(`CodeEngine cannot be used after it has been disposed.`);
        }
    }
    /**
     * Handles unexpected errors, such as worker threads crashing.
     * @internal
     */
    async _crash(error) {
        try {
            this.emit("error" /* Error */, error);
        }
        finally {
            await this.dispose();
        }
    }
}
exports.CodeEngine = CodeEngine;
//# sourceMappingURL=code-engine.js.map