"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const file_path_filter_1 = require("file-path-filter");
/**
 * Wraps a plugin's functionality in a consistent interface.
 * @internal
 */
class PluginController {
    constructor(plugin) {
        this.name = plugin.name;
        this.filter = file_path_filter_1.createFilter({ map }, plugin.filter === undefined ? true : plugin.filter);
        this._plugin = plugin;
        plugin.processFile || (this.processFile = undefined);
        plugin.processFiles || (this.processFiles = undefined);
        plugin.read || (this.read = undefined);
        plugin.watch || (this.watch = undefined);
        plugin.clean || (this.clean = undefined);
        plugin.dispose || (this.dispose = undefined);
    }
    // tslint:disable-next-line: no-async-without-await
    async *processFiles(files, context) {
        try {
            let fileInfos = this._plugin.processFiles(files, context);
            for await (let fileInfo of utils_1.iterate(fileInfos)) {
                yield utils_1.createFile(fileInfo, this.name);
            }
            // Ensure that all input files are read, even if the plugin doesn't actually read them.
            // Otherwise the build will never complete.
            await utils_1.drainIterable(files);
        }
        catch (error) {
            throw utils_1.typedOno(error, `An error occurred in ${this} while processing files.`);
        }
    }
    async processFile(file, context, output) {
        try {
            let fileInfos = this._plugin.processFile(file, context);
            for await (let fileInfo of utils_1.iterate(fileInfos)) {
                await output.write(utils_1.createFile(fileInfo, this.name));
            }
        }
        catch (error) {
            throw utils_1.typedOno(error, `An error occurred in ${this} while processing ${file}.`);
        }
    }
    read(context) {
        let iterator;
        try {
            // NOTE: The reason we don't just use a `for await...of` loop here is that we want to be able to
            // ead multiple files simultaneously if the `processFile()` plugins are faster than the IO source.
            let fileInfos = this._plugin.read(context);
            iterator = utils_1.iterate(fileInfos)[Symbol.asyncIterator]();
        }
        catch (error) {
            throw utils_1.typedOno(error, `An error occurred in ${this} while reading source files.`);
        }
        let next = async () => {
            try {
                let result = await iterator.next();
                if (result.done) {
                    return result;
                }
                let file = utils_1.createFile(result.value, this.name);
                return { value: file };
            }
            catch (error) {
                throw utils_1.typedOno(error, `An error occurred in ${this} while reading source files.`);
            }
        };
        return { [Symbol.asyncIterator]: () => ({ next }) };
    }
    // tslint:disable-next-line: no-async-without-await
    async *watch(context) {
        try {
            let changedFileInfos = this._plugin.watch(context);
            for await (let changedFileInfo of utils_1.iterate(changedFileInfos)) {
                yield utils_1.createChangedFile(changedFileInfo, this.name);
            }
        }
        catch (error) {
            throw utils_1.typedOno(error, `An error occurred in ${this} while watching source files for changes.`);
        }
    }
    async clean(context) {
        try {
            await this._plugin.clean(context);
        }
        catch (error) {
            throw utils_1.typedOno(error, `An error occurred in ${this} while cleaning the destination.`);
        }
    }
    async dispose(context) {
        try {
            await this._plugin.dispose(context);
        }
        catch (error) {
            throw utils_1.typedOno(error, `An error occurred in ${this} while cleaning-up.`);
        }
    }
    /**
     * Returns a string representation of the plugin.
     */
    toString() {
        return this.name;
    }
}
exports.PluginController = PluginController;
/**
 * Maps `File` objects to paths for filtering
 */
function map(file) {
    return file.path;
}
//# sourceMappingURL=plugin-controller.js.map